var running = false;
const ws = new WebSocket('ws://' + location.host + ':8080');
ws.binaryType = 'arraybuffer';

var errorCounter = 0;

var idn = "";
var freq = 0;
var pow = 0;
var phas = 0;
var outpStat = 0;
var extFreq = 0;
var lock = 0;
var refSour = "";
var refOutp = "";
var refOutpFreq = 0;
var ip = "";
var subn = "";
var gate = "";
var dhcp = "";

var specificStatusCodeMappings = {
    '1000': 'Normal Closure',
    '1001': 'Going Away',
    '1002': 'Protocol Error',
    '1003': 'Unsupported Data',
    '1004': '(For future)',
    '1005': 'No Status Received',
    '1006': 'Abnormal Closure: Check if an other client is connected.',
    '1007': 'Invalid frame payload data',
    '1008': 'Policy Violation',
    '1009': 'Message too big',
    '1010': 'Missing Extension',
    '1011': 'Internal Error',
    '1012': 'Service Restart',
    '1013': 'Try Again Later',
    '1014': 'Bad Gateway',
    '1015': 'TLS Handshake'
};

var waiting = false;

var recieved = false;

var currentResponse = "";

ws.onclose = (event) => {
    if(event.wasClean){
        postMessage("Disconnected;");
    } else {
        postMessage("Close;Disconnected: " + event.code + " - " + specificStatusCodeMappings[event.code]);
    }
    console.log('disconnected');
}

ws.onerror = (e) => {
    errorCounter++;
    if(errorCounter === 5) {
        ws.close();
    }
}

ws.addEventListener("message", e => {
    waiting = false;
    var decoder = new TextDecoder('UTF-8');
    var encoder = new TextEncoder();
    var data = decoder.decode(e.data);
    data.trim();
    recieved = true;
    if ((e.data + "").startsWith("QRY")) {
        postMessage(lastCommand + ";" + data);
    } else if (lastCommand === "*IDN?") {
        if (data !== idn) {
            idn = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "FREQ?") {
        if (parseInt(data) !== freq) {
            freq = parseInt(data);
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "OUTP:STAT?") {
        if (parseInt(data) !== outpStat) {
            outpStat = parseInt(data);
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "FREQ") {
        lastCommand = "FREQ?";
    } else if (lastCommand === "OUTP:STAT") {
        lastCommand = "OUTP:STAT?";
    } else if (lastCommand === "POW") {
        lastCommand = "POW?";
    } else if (lastCommand === "POW?") {
        if (parseInt(data) !== pow) {
            pow = parseInt(data);
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "PHAS") {
        lastCommand = "PHAS?";
    } else if (lastCommand === "PHAS?") {
        if (parseInt(data) !== phas) {
            phas = parseInt(data);
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "ROSC:EXT:FREQ?") {
        if (parseInt(data) !== extFreq) {
            extFreq = parseInt(data);
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "ROSC:LOCK?") {
        if (data !== lock) {
            lock = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "ROSC:SOUR?") {
        if (data !== refSour) {
            refSour = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "ROSC:OUTP?") {
        if (data !== refOutp) {
            refOutp = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "ROSC:OUTP:FREQ?") {
        if (data !== refOutpFreq) {
            refOutpFreq = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "SYST:COMM:LAN:IP?") {
        if (data !== ip) {
            ip = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "SYST:COMM:LAN:SUBN?") {
        if (data !== subn) {
            subn = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "SYST:COMM:LAN:GATE?") {
        if (data !== gate) {
            gate = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if (lastCommand === "SYST:COMM:LAN:CONF?") {
        if (data !== dhcp) {
            dhcp = data;
            postMessage(lastCommand + ";" + data);
        }
    } else if(lastCommand.startsWith("RESET")){
        postMessage(lastCommand.replace("RESET:", "") + ";" + data);
    } else {
        postMessage(lastCommand + ";" + data);
    }
});

ws.onopen = () => {
    console.log('connected');
    var encoder = new TextEncoder();
    lastCommand = "FREQ?MAX";
    ws.send(encoder.encode("FREQ?MAX\n"));
    setTimeout(function () { lastCommand = "POW?MAX"; ws.send(encoder.encode("POW?MAX\n")); }, 30);
    setTimeout(function () { lastCommand = "ROSC:EXT:FREQ?MAX"; ws.send(encoder.encode("ROSC:EXT:FREQ?MAX\n")); }, 60);
    setTimeout(function () { lastCommand = "POW?MIN"; ws.send(encoder.encode("POW?MIN\n")); }, 90);
    setTimeout(function () { lastCommand = "ROSC:EXT:FREQ?MIN"; ws.send(encoder.encode("ROSC:EXT:FREQ?MIN\n")); }, 120);
    setTimeout(function () { lastCommand = "FREQ?MIN"; ws.send(encoder.encode("FREQ?MIN\n")); }, 150);
    setTimeout(() => readData(), 200);
}

onclose = () => {
    console.log('closing...');
    ws.close();
}

onmessage = (e) => {
    if (e.data === "Refresh") {
        refresh();
    } else if(e.data === "ResetSystemSettings") {
        resetSystemSettings();
    } else {
        lastCommand = e.data.split(" ")[0].trim();
        var data = e.data.trim().replace("QRY", "").replace("RESET:", "");
        var encoder = new TextEncoder();
        ws.send(encoder.encode(data + "\n"));
    }
}

function readData() {
    waiting = true;
    var encoder = new TextEncoder();
    lastCommand = "*IDN?";
    ws.send(encoder.encode("*IDN?\n"));
    setTimeout(function () { lastCommand = "FREQ?"; ws.send(encoder.encode("FREQ?\n")); }, 30);
    setTimeout(function () { lastCommand = "POW?"; ws.send(encoder.encode("POW?\n")); }, 60);
    setTimeout(function () { lastCommand = "OUTP:STAT?"; ws.send(encoder.encode("OUTP:STAT?\n")); }, 90);
    setTimeout(function () { lastCommand = "PHAS?"; ws.send(encoder.encode("PHAS?\n")); }, 120);
    setTimeout(function () { lastCommand = "ROSC:EXT:FREQ?"; ws.send(encoder.encode("ROSC:EXT:FREQ?\n")); }, 150);
    setTimeout(function () { lastCommand = "ROSC:LOCK?"; ws.send(encoder.encode("ROSC:LOCK?\n")); }, 180);
    setTimeout(function () { lastCommand = "ROSC:SOUR?"; ws.send(encoder.encode("ROSC:SOUR?\n")); }, 210);
    setTimeout(function () { lastCommand = "ROSC:OUTP?"; ws.send(encoder.encode("ROSC:OUTP?\n")); }, 240);
    setTimeout(function () { lastCommand = "ROSC:OUTP:FREQ?"; ws.send(encoder.encode("ROSC:OUTP:FREQ?\n")); }, 270);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:IP?"; ws.send(encoder.encode("SYST:COMM:LAN:IP?\n")); }, 300);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:SUBN?"; ws.send(encoder.encode("SYST:COMM:LAN:SUBN?\n")); }, 330);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:GATE?"; ws.send(encoder.encode("SYST:COMM:LAN:GATE?\n")); }, 360);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:CONF?"; ws.send(encoder.encode("SYST:COMM:LAN:CONF?\n")); }, 390);

    //checkIfDataReturned();
    setTimeout(() => {
        readData();
    }, 3000);

}

function checkIfDataReturned() {
    if(waiting){
        postMessage("Disconnected;");
        ws.close();
    }
}

function refresh() {
    waiting = true;
    var encoder = new TextEncoder();
    lastCommand = "*IDN?";
    ws.send(encoder.encode("*IDN?\n"));
    setTimeout(function () { lastCommand = "FREQ?"; ws.send(encoder.encode("FREQ?\n")); }, 30);
    setTimeout(function () { lastCommand = "POW?"; ws.send(encoder.encode("POW?\n")); }, 60);
    setTimeout(function () { lastCommand = "OUTP:STAT?"; ws.send(encoder.encode("OUTP:STAT?\n")); }, 90);
    setTimeout(function () { lastCommand = "PHAS?"; ws.send(encoder.encode("PHAS?\n")); }, 120);
    setTimeout(function () { lastCommand = "ROSC:EXT:FREQ?"; ws.send(encoder.encode("ROSC:EXT:FREQ?\n")); }, 150);
    setTimeout(function () { lastCommand = "ROSC:LOCK?"; ws.send(encoder.encode("ROSC:LOCK?\n")); }, 180);
    setTimeout(function () { lastCommand = "ROSC:SOUR?"; ws.send(encoder.encode("ROSC:SOUR?\n")); }, 210);
    setTimeout(function () { lastCommand = "ROSC:OUTP?"; ws.send(encoder.encode("ROSC:OUTP?\n")); }, 240);
    setTimeout(function () { lastCommand = "ROSC:OUTP:FREQ?"; ws.send(encoder.encode("ROSC:OUTP:FREQ?\n")); }, 270);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:IP?"; ws.send(encoder.encode("SYST:COMM:LAN:IP?\n")); }, 300);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:SUBN?"; ws.send(encoder.encode("SYST:COMM:LAN:SUBN?\n")); }, 330);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:GATE?"; ws.send(encoder.encode("SYST:COMM:LAN:GATE?\n")); }, 360);
    setTimeout(function () { lastCommand = "SYST:COMM:LAN:CONF?"; ws.send(encoder.encode("SYST:COMM:LAN:CONF?\n")); }, 390);
}

function resetSystemSettings() {
    var encoder = new TextEncoder();
    setTimeout(function () { lastCommand = "RESET:SYST:COMM:LAN:IP?"; ws.send(encoder.encode("SYST:COMM:LAN:IP?\n")); }, 0);
    setTimeout(function () { lastCommand = "RESET:SYST:COMM:LAN:SUBN?"; ws.send(encoder.encode("SYST:COMM:LAN:SUBN?\n")); }, 30);
    setTimeout(function () { lastCommand = "RESET:SYST:COMM:LAN:GATE?"; ws.send(encoder.encode("SYST:COMM:LAN:GATE?\n")); }, 60);
    setTimeout(function () { lastCommand = "RESET:SYST:COMM:LAN:CONF?"; ws.send(encoder.encode("SYST:COMM:LAN:CONF?\n")); }, 90);
}